@@ -1,7 +1,7 @@
 # Declare .data as global variable to avoid R CMD checks.
 utils::globalVariables(c(".data",
-                       "..x..",
-                       "..quantile..",
-                       "..ecdf..",
+                       "x",
+                       "quantile",
+                       "ecdf",
                        "summarise",
                        "stratum"))
